@@ -13,8 +13,11 @@
 ##############################################################################
 """Test Interface implementation
 """
+# pylint:disable=protected-access
 import unittest
 
+from zope.interface._compat import _skip_under_py3k
+
 _marker = object()
 
 
@@ -25,7 +28,7 @@ class Test_invariant(unittest.TestCase):
         from zope.interface.interface import TAGGED_DATA
 
         def _check(*args, **kw):
-            pass
+            raise NotImplementedError()
 
         class Foo(object):
             invariant(_check)
@@ -38,10 +41,10 @@ class Test_invariant(unittest.TestCase):
         from zope.interface.interface import TAGGED_DATA
 
         def _check(*args, **kw):
-            pass
+            raise NotImplementedError()
 
         def _another_check(*args, **kw):
-            pass
+            raise NotImplementedError()
 
         class Foo(object):
             invariant(_check)
@@ -95,12 +98,10 @@ class ElementTests(unittest.TestCase):
         from zope.interface.interface import Element
         return Element
 
-    def _makeOne(self,  name=None, __doc__=_marker):
+    def _makeOne(self,  name=None):
         if name is None:
             name = self.DEFAULT_NAME
-        if __doc__ is _marker:
-            return self._getTargetClass()(name)
-        return self._getTargetClass()(name, __doc__)
+        return self._getTargetClass()(name)
 
     def test_ctor_defaults(self):
         element = self._makeOne()
@@ -184,27 +185,43 @@ class SpecificationBasePyTests(unittest.TestCase):
 
     def test_isOrExtends_miss(self):
         sb = self._makeOne()
-        sb._implied = {}  # not defined by SpecificationBasePy 
+        sb._implied = {}  # not defined by SpecificationBasePy
         self.assertFalse(sb.isOrExtends(object()))
 
     def test_isOrExtends_hit(self):
         sb = self._makeOne()
         testing = object()
-        sb._implied = {testing: {}}  # not defined by SpecificationBasePy 
+        sb._implied = {testing: {}}  # not defined by SpecificationBasePy
         self.assertTrue(sb(testing))
 
     def test___call___miss(self):
         sb = self._makeOne()
-        sb._implied = {}  # not defined by SpecificationBasePy 
+        sb._implied = {}  # not defined by SpecificationBasePy
         self.assertFalse(sb.isOrExtends(object()))
 
     def test___call___hit(self):
         sb = self._makeOne()
         testing = object()
-        sb._implied = {testing: {}}  # not defined by SpecificationBasePy 
+        sb._implied = {testing: {}}  # not defined by SpecificationBasePy
         self.assertTrue(sb(testing))
 
 
+class SpecificationBaseTests(unittest.TestCase):
+
+    def _getTargetClass(self):
+        from zope.interface.interface import SpecificationBase
+        return SpecificationBase
+
+    def test_optimizations(self):
+        from zope.interface.interface import SpecificationBasePy
+        try:
+            import zope.interface._zope_interface_coptimizations
+        except ImportError:
+            self.assertIs(self._getTargetClass(), SpecificationBasePy)
+        else:
+            self.assertIsNot(self._getTargetClass(), SpecificationBasePy)
+
+
 class InterfaceBasePyTests(unittest.TestCase):
 
     def _getTargetClass(self):
@@ -266,6 +283,22 @@ class InterfaceBasePyTests(unittest.TestCase):
             self.assertEqual(_missed, [(ib, adapted)])
 
 
+class InterfaceBaseTests(unittest.TestCase):
+
+    def _getTargetClass(self):
+        from zope.interface.interface import InterfaceBase
+        return InterfaceBase
+
+    def test_optimizations(self):
+        from zope.interface.interface import InterfaceBasePy
+        try:
+            import zope.interface._zope_interface_coptimizations
+        except ImportError:
+            self.assertIs(self._getTargetClass(), InterfaceBasePy)
+        else:
+            self.assertIsNot(self._getTargetClass(), InterfaceBasePy)
+
+
 class SpecificationTests(unittest.TestCase):
 
     def _getTargetClass(self):
@@ -795,16 +828,14 @@ class InterfaceClassTests(unittest.TestCase):
 
     def test___hash___missing_required_attrs(self):
         import warnings
-        try:
-            from warnings import catch_warnings
-        except ImportError:  # Python 2.5
-            return
+        from warnings import catch_warnings
+
         class Derived(self._getTargetClass()):
             def __init__(self):
                 pass # Don't call base class.
         derived = Derived()
         with catch_warnings(record=True) as warned:
-            warnings.simplefilter('always') # see LP #825249 
+            warnings.simplefilter('always') # see LP #825249
             self.assertEqual(hash(derived), 1)
             self.assertEqual(len(warned), 1)
             self.assertTrue(warned[0].category is UserWarning)
@@ -873,7 +904,7 @@ class InterfaceTests(unittest.TestCase):
         class I1(Interface):
 
             def method(foo, bar, bingo):
-                pass
+                "A method"
 
         self.assertTrue(I1['method'].interface is I1)
 
@@ -894,9 +925,9 @@ class InterfaceTests(unittest.TestCase):
         class Current(object):
             __implemented__ = ICurrent
             def method1(self, a, b):
-                return 1
+                raise NotImplementedError()
             def method2(self, a, b):
-                return 2
+                raise NotImplementedError()
 
         current = Current()
 
@@ -919,7 +950,7 @@ class InterfaceTests(unittest.TestCase):
         class Current():
             __implemented__ = IBase
             def method(self):
-                pass
+                raise NotImplementedError()
         current = Current()
 
         self.assertTrue(IBase.implementedBy(Current))
@@ -944,7 +975,7 @@ class InterfaceTests(unittest.TestCase):
         class Current(object):
             __implemented__ = IDerived
             def method(self):
-                pass
+                raise NotImplementedError()
 
         current = Current()
 
@@ -973,7 +1004,7 @@ class InterfaceTests(unittest.TestCase):
             __implemented__ = ILeft
 
             def method(self):
-                pass
+                raise NotImplementedError()
 
         class Right(object):
             __implemented__ = IRight
@@ -1010,7 +1041,7 @@ class InterfaceTests(unittest.TestCase):
             __implemented__ = ILeft
 
             def method(self):
-                pass
+                raise NotImplementedError()
 
         class Right(object):
             __implemented__ = IRight
@@ -1054,20 +1085,20 @@ class InterfaceTests(unittest.TestCase):
         from zope.interface import Attribute
         from zope.interface import Interface
         from zope.interface.verify import verifyClass
-        from zope.interface._compat import _u
+
 
         class ICheckMe(Interface):
-            attr = Attribute(_u('My attr'))
+            attr = Attribute(u'My attr')
 
             def method():
-                pass
+                "A method"
 
         class CheckMe(object):
             __implemented__ = ICheckMe
             attr = 'value'
 
             def method(self):
-                pass
+                raise NotImplementedError()
 
         self.assertTrue(verifyClass(ICheckMe, CheckMe))
 
@@ -1075,20 +1106,20 @@ class InterfaceTests(unittest.TestCase):
         from zope.interface import Attribute
         from zope.interface import Interface
         from zope.interface.verify import verifyObject
-        from zope.interface._compat import _u
+
 
         class ICheckMe(Interface):
-            attr = Attribute(_u('My attr'))
+            attr = Attribute(u'My attr')
 
             def method():
-                pass
+                "A method"
 
         class CheckMe(object):
             __implemented__ = ICheckMe
             attr = 'value'
 
             def method(self):
-                pass
+                raise NotImplementedError()
 
         check_me = CheckMe()
 
@@ -1105,10 +1136,10 @@ class InterfaceTests(unittest.TestCase):
     def test_names_simple(self):
         from zope.interface import Attribute
         from zope.interface import Interface
-        from zope.interface._compat import _u
+
 
         class ISimple(Interface):
-            attr = Attribute(_u('My attr'))
+            attr = Attribute(u'My attr')
 
             def method():
                 pass
@@ -1118,16 +1149,16 @@ class InterfaceTests(unittest.TestCase):
     def test_names_derived(self):
         from zope.interface import Attribute
         from zope.interface import Interface
-        from zope.interface._compat import _u
+
 
         class IBase(Interface):
-            attr = Attribute(_u('My attr'))
+            attr = Attribute(u'My attr')
 
             def method():
                 pass
 
         class IDerived(IBase):
-            attr2 = Attribute(_u('My attr2'))
+            attr2 = Attribute(u'My attr2')
 
             def method():
                 pass
@@ -1144,10 +1175,10 @@ class InterfaceTests(unittest.TestCase):
         from zope.interface import Attribute
         from zope.interface.interface import Method
         from zope.interface import Interface
-        from zope.interface._compat import _u
+
 
         class ISimple(Interface):
-            attr = Attribute(_u('My attr'))
+            attr = Attribute(u'My attr')
 
             def method():
                 "My method"
@@ -1168,16 +1199,16 @@ class InterfaceTests(unittest.TestCase):
         from zope.interface import Attribute
         from zope.interface import Interface
         from zope.interface.interface import Method
-        from zope.interface._compat import _u
+
 
         class IBase(Interface):
-            attr = Attribute(_u('My attr'))
+            attr = Attribute(u'My attr')
 
             def method():
                 "My method"
 
         class IDerived(IBase):
-            attr2 = Attribute(_u('My attr2'))
+            attr2 = Attribute(u'My attr2')
 
             def method():
                 "My method, overridden"
@@ -1233,10 +1264,10 @@ class InterfaceTests(unittest.TestCase):
         from zope.interface import Attribute
         from zope.interface.interface import Method
         from zope.interface import Interface
-        from zope.interface._compat import _u
+
 
         class ISimple(Interface):
-            attr = Attribute(_u('My attr'))
+            attr = Attribute(u'My attr')
 
             def method():
                 "My method"
@@ -1255,16 +1286,16 @@ class InterfaceTests(unittest.TestCase):
         from zope.interface import Attribute
         from zope.interface.interface import Method
         from zope.interface import Interface
-        from zope.interface._compat import _u
+
 
         class IBase(Interface):
-            attr = Attribute(_u('My attr'))
+            attr = Attribute(u'My attr')
 
             def method():
                 "My method"
 
         class IDerived(IBase):
-            attr2 = Attribute(_u('My attr2'))
+            attr2 = Attribute(u'My attr2')
 
             def method():
                 "My method, overridden"
@@ -1304,10 +1335,10 @@ class InterfaceTests(unittest.TestCase):
         from zope.interface import Attribute
         from zope.interface.interface import Method
         from zope.interface import Interface
-        from zope.interface._compat import _u
+
 
         class ISimple(Interface):
-            attr = Attribute(_u('My attr'))
+            attr = Attribute(u'My attr')
 
             def method():
                 "My method"
@@ -1326,16 +1357,16 @@ class InterfaceTests(unittest.TestCase):
         from zope.interface import Attribute
         from zope.interface.interface import Method
         from zope.interface import Interface
-        from zope.interface._compat import _u
+
 
         class IBase(Interface):
-            attr = Attribute(_u('My attr'))
+            attr = Attribute(u'My attr')
 
             def method():
                 "My method"
 
         class IDerived(IBase):
-            attr2 = Attribute(_u('My attr2'))
+            attr2 = Attribute(u'My attr2')
 
             def method():
                 "My method, overridden"
@@ -1374,10 +1405,10 @@ class InterfaceTests(unittest.TestCase):
     def test___contains__simple(self):
         from zope.interface import Attribute
         from zope.interface import Interface
-        from zope.interface._compat import _u
+
 
         class ISimple(Interface):
-            attr = Attribute(_u('My attr'))
+            attr = Attribute(u'My attr')
 
             def method():
                 "My method"
@@ -1388,16 +1419,16 @@ class InterfaceTests(unittest.TestCase):
     def test___contains__derived(self):
         from zope.interface import Attribute
         from zope.interface import Interface
-        from zope.interface._compat import _u
+
 
         class IBase(Interface):
-            attr = Attribute(_u('My attr'))
+            attr = Attribute(u'My attr')
 
             def method():
                 "My method"
 
         class IDerived(IBase):
-            attr2 = Attribute(_u('My attr2'))
+            attr2 = Attribute(u'My attr2')
 
             def method():
                 "My method, overridden"
@@ -1421,10 +1452,10 @@ class InterfaceTests(unittest.TestCase):
     def test___iter__simple(self):
         from zope.interface import Attribute
         from zope.interface import Interface
-        from zope.interface._compat import _u
+
 
         class ISimple(Interface):
-            attr = Attribute(_u('My attr'))
+            attr = Attribute(u'My attr')
 
             def method():
                 "My method"
@@ -1434,16 +1465,16 @@ class InterfaceTests(unittest.TestCase):
     def test___iter__derived(self):
         from zope.interface import Attribute
         from zope.interface import Interface
-        from zope.interface._compat import _u
+
 
         class IBase(Interface):
-            attr = Attribute(_u('My attr'))
+            attr = Attribute(u'My attr')
 
             def method():
                 "My method"
 
         class IDerived(IBase):
-            attr2 = Attribute(_u('My attr2'))
+            attr2 = Attribute(u'My attr2')
 
             def method():
                 "My method, overridden"
@@ -1491,11 +1522,10 @@ class InterfaceTests(unittest.TestCase):
         e = []
         try:
             iface.validateInvariants(has_invariant, e)
+            self.fail("validateInvariants should always raise")
         except Invalid as error:
             self.assertEqual(error.args[0], e)
-        else:
-            self._assert(0) # validateInvariants should always raise
-            # Invalid
+
         self.assertEqual(len(e), error_len)
         msgs = [error.args[0] for error in e]
         msgs.sort()
@@ -1649,21 +1679,22 @@ class InterfaceTests(unittest.TestCase):
         self.assertEqual(I.__doc__, "")
         self.assertEqual(list(I), ['__doc__'])
 
+    @_skip_under_py3k
     def testIssue228(self):
         # Test for http://collector.zope.org/Zope3-dev/228
         # Old style classes don't have a '__class__' attribute
+        # No old style classes in Python 3, so the test becomes moot.
         import sys
-        if sys.version[0] < '3':
-            # No old style classes in Python 3, so the test becomes moot.
-            from zope.interface import Interface
 
-            class I(Interface):
-                "xxx"
+        from zope.interface import Interface
 
-            class OldStyle:
-                __providedBy__ = None
+        class I(Interface):
+            "xxx"
 
-            self.assertRaises(AttributeError, I.providedBy, OldStyle)
+        class OldStyle:
+            __providedBy__ = None
+
+        self.assertRaises(AttributeError, I.providedBy, OldStyle)
 
     def test_invariant_as_decorator(self):
         from zope.interface import Interface
@@ -1675,7 +1706,7 @@ class InterfaceTests(unittest.TestCase):
         class IRange(Interface):
             min = Attribute("Lower bound")
             max = Attribute("Upper bound")
-            
+
             @invariant
             def range_invariant(ob):
                 if ob.max < ob.min:
@@ -2054,8 +2085,8 @@ def _barGreaterThanFoo(obj):
     foo = getattr(obj, 'foo', None)
     bar = getattr(obj, 'bar', None)
     if foo is not None and isinstance(foo, type(bar)):
-        # type checking should be handled elsewhere (like, say, 
-        # schema); these invariants should be intra-interface 
+        # type checking should be handled elsewhere (like, say,
+        # schema); these invariants should be intra-interface
         # constraints.  This is a hacky way to do it, maybe, but you
         # get the idea
         if not bar > foo:
@@ -2081,20 +2112,3 @@ class _Monkey(object):
     def __exit__(self, exc_type, exc_val, exc_tb):
         for key, value in self.to_restore.items():
             setattr(self.module, key, value)
-
-
-def test_suite():
-    import doctest
-    return unittest.TestSuite((
-        unittest.makeSuite(ElementTests),
-        unittest.makeSuite(SpecificationBasePyTests),
-        unittest.makeSuite(InterfaceBasePyTests),
-        unittest.makeSuite(SpecificationTests),
-        unittest.makeSuite(InterfaceTests),
-        unittest.makeSuite(AttributeTests),
-        unittest.makeSuite(MethodTests),
-        unittest.makeSuite(Test_fromFunction),
-        #unittest.makeSuite(Test_fromMethod),
-        doctest.DocTestSuite(),
-        doctest.DocTestSuite("zope.interface.interface"),
-    ))
